import makeEventHandler from "./events.js";

export default el => {
  if (!(el instanceof HTMLElement)) {
    throw "el must be a HTMLElement";
  }

  const events = makeEventHandler();
  const state = {
    dragging: false,
    rightClick: false,
  };

  el.addEventListener("mousedown", () => {
    document.addEventListener("mousemove", onDrag);
    document.addEventListener("mouseup", onMouseUp);
  });

  function onDrag(e) {
    state.dragging = true;
    document.body.style.cursor = "move";
    events.emit("drag", {
      dx: e.movementX * window.devicePixelRatio,
      dy: e.movementY * window.devicePixelRatio,
      keys: keys(e),
    });
  }
  function onMouseUp(e) {
    document.removeEventListener("mousemove", onDrag);
    document.body.style.cursor = null;
    if (!state.dragging) {
      events.emit(state.rightClick ? "right-click" : "click", {
        position: position(e),
        keys: keys(e),
      });
    }
    state.dragging = false;
    state.rightClick = false;
  }

  el.addEventListener("wheel", e => {
    e.preventDefault();
    events.emit("wheel", {
      position: position(e),
      dx: e.deltaX,
      dy: e.deltaY,
      keys: keys(e),
    });
  });

  el.addEventListener("contextmenu", e => {
    e.preventDefault();
    state.rightClick = true;
  });

  el.addEventListener("mousemove", e => {
    if (!state.dragging) {
      events.emit("move", { position: position(e), keys: keys(e) });
    }
  });

  window.addEventListener("keydown", e => {
    if (e.target.tagName === "INPUT" || e.target.tagName === "TEXTAREA") return;
    e.preventDefault();
    switch (event.key) {
      case "ArrowDown": {
        events.emit("arrow-down", { keys: keys(e) });
        break;
      }
      case "ArrowUp":
        events.emit("arrow-up", { keys: keys(e) });
        break;
      case "ArrowLeft":
        events.emit("arrow-left", { keys: keys(e) });
        break;
      case "ArrowRight":
        events.emit("arrow-right", { keys: keys(e) });
        break;
    }
  });

  return { ...events };
};

function keys(event = {}) {
  return {
    ctrl: event.ctrlKey,
    alt: event.altKey,
    shift: event.shiftKey,
    meta: event.metaKey,
  };
}

function position(event = {}) {
  return {
    x: event.offsetX * window.devicePixelRatio,
    y: event.offsetY * window.devicePixelRatio,
  };
}
